/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [https://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package org.neo4j.cypher.internal

import org.neo4j.cypher.internal.parser.common.ast.factory.ASTExceptionFactory
import org.neo4j.cypher.internal.util.CypherExceptionFactory
import org.neo4j.cypher.internal.util.InputPosition
import org.neo4j.gqlstatus.ErrorGqlStatusObject
import org.neo4j.gqlstatus.GqlHelper

import java.util

import scala.collection.convert.AsScalaConverters

// TODO Remove, use same as antlr parser
class Neo4jASTExceptionFactory(inner: CypherExceptionFactory) extends ASTExceptionFactory with AsScalaConverters {

  override def syntaxException(
    got: String,
    expected: util.List[String],
    source: Exception,
    offset: Int,
    line: Int,
    column: Int
  ): Exception = {
    val exp: Seq[String] = asScala(expected).toSeq

    val message =
      new StringBuilder("Invalid input '")
        .append(got)
        .append("':")
        .append(" expected").append(
          if (exp.size == 1)
            " " + exp.head
          else if (exp.size < 5)
            " " + exp.init.mkString(", ") + " or " + exp.last
          else
            System.lineSeparator() + exp.map("  " + _).mkString(System.lineSeparator())
        ).result()

    inner.syntaxException(
      GqlHelper.getGql42001_42I06(got, expected, offset, line, column),
      message,
      InputPosition(offset, line, column)
    )
  }

  override def syntaxException(
    gqlStatusObject: ErrorGqlStatusObject,
    got: String,
    expected: util.List[String],
    source: Exception,
    offset: Int,
    line: Int,
    column: Int
  ): Exception = {
    val exp: Seq[String] = asScala(expected).toSeq

    val message =
      new StringBuilder("Invalid input '")
        .append(got)
        .append("':")
        .append(" expected").append(
          if (exp.size == 1)
            " " + exp.head
          else if (exp.size < 5)
            " " + exp.init.mkString(", ") + " or " + exp.last
          else
            System.lineSeparator() + exp.map("  " + _).mkString(System.lineSeparator())
        ).result()

    inner.syntaxException(
      GqlHelper.getGql42001_42I06_withCause(got, expected, offset, line, column, gqlStatusObject),
      message,
      InputPosition(offset, line, column)
    )
  }

  override def syntaxException(
    gqlStatusObject: ErrorGqlStatusObject,
    source: Exception,
    offset: Int,
    line: Int,
    column: Int
  ): Exception =
    inner.syntaxException(gqlStatusObject, source.getMessage, InputPosition(offset, line, column))

  override def syntaxException(source: Exception, offset: Int, line: Int, column: Int): Exception =
    inner.syntaxException(source.getMessage, InputPosition(offset, line, column))
}
